from gramag import MagGraph, format_rank_table, DirectSum

# Create graph (tuple of edges, no support for isolated vertices atm)
mg = MagGraph([(0, 1), (0, 2), (0, 3), (1, 6), (2, 6), (3, 4), (4, 5), (5, 6)])
# Compute generators of all MC^{(s, t)}_{k, l} for l<=l_max
l_max = 6
mg.populate_paths(l_max=l_max)

# This is just counting, should be quick
rk_gens = mg.rank_generators()

# For each l, in parallel across each (s, t), computes MH^{(s, t)}_{k, l}
# Adds up the rank for each k, l
rk_hom = mg.rank_homology()

# Pretty print
print("Rank of MC:")
print(format_rank_table(rk_gens))

print("Rank of MH:")
print(format_rank_table(rk_hom))


# Count generators for a given list (s, t)
print("Rank of MC^{(0, 6)}:")
print(format_rank_table(mg.rank_generators([(0, 6)])))

for s in range(6):
    for t in range(6):
        # Ignoring l<=1 because they're usually boring
        for l in range(2, 6):
            # Gets Magnitude homology for fixed (s, t), l (and all k<=l)
            # Will return None if l > l_max
            # Final boolean parameter determines whether representatives should be computed
            hom = mg.stl_homology((s, t), l, representatives=True)
            total_rank = sum([rk for rk in hom.ranks.values()])
            if total_rank == 0:
                continue
            print(f"({(s, t)}, l={l})")
            print(hom.ranks)
            print(hom.representatives)
            print("")


print("l=2")
l2_hom = mg.l_homology(2, representatives=True)
print(l2_hom.ranks)
print(l2_hom.representatives)
print("")

print("l=4")
l2_hom = mg.l_homology(4, representatives=True)
print(l2_hom.ranks)
print(l2_hom.representatives)
print("")


print("Custom direct sum:")
ds = DirectSum(
    [
        mg.stl_homology((s, t), l, representatives=True)
        for s in range(7)
        for t in range(7)
        for l in range(2, 7)
    ]
)
print(ds.ranks)
print(ds.representatives)
print("")

print("Fixed l=1, custom (s, t) list - computed in parallel!")
ds2 = mg.l_homology(1, representatives=True, node_pairs=[(s, 6) for s in range(7)])
print(ds2.ranks)
print(ds2.representatives)
print("")

print("Errors:")
try:
    mg.l_homology(12)
except TypeError as e:
    print(e)

try:
    mg.l_homology(2, representatives=False).representatives
except TypeError as e:
    print(e)

try:
    mg.populate_paths(k_max=5, l_max=5)
except TypeError as e:
    print(e)

# No error because no homology
print(mg.l_homology(5, representatives=False).representatives)
print("")


print("Stopping condition based on k_max:")
# No limit on l, but k_max up to 3 so can compute homology up to k=2
mg.populate_paths(k_max=3)
print("MC:")
print(format_rank_table(mg.rank_generators()))
print("MH:")
print(format_rank_table(mg.rank_homology(), zero="0", unknown="-", above_diagonal="0"))
print("l=4 representatives")
print(mg.l_homology(4, representatives=True).representatives)



from gramag import MagGraph, format_rank_table

# Create your graph
# A few rules:
# 1. Nodes are labelled by integers
# 2. Edges are provided as a list of tuples of vertices
# 3. Isolated vertices are not supported at the moment
mg = MagGraph([(0, 1), (0, 2), (0, 3), (1, 6), (2, 6), (3, 4), (4, 5), (5, 6)])

# Compute generators of all MC^{(s, t)}_{k, l} for l<=6
mg.populate_paths(l_max=6)

# Reports the ranks of MC^{(s, t)}_{k, l}, summed over (s, t)
rk_gens = mg.rank_generators()

# For each l, in parallel across each (s, t), computes MH^{(s, t)}_{k, l}
# Adds up the rank for each k, l
rk_hom = mg.rank_homology()

# Pretty print
print("Rank of MC:")
print(format_rank_table(rk_gens))

print("Rank of MH:")
print(format_rank_table(rk_hom))

# Compute homology summed over a given list of (s, t)
print("Rank of MH^{(0, 6)}:")
print(format_rank_table(mg.rank_homology(node_pairs=[(0, 6)])))

# Compute homology with representatives, at a given l
homology = mg.l_homology(4, representatives=True)
print("Representatives for MH_{2, 4}:")
print(homology.representatives[2])



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

import gramag

project = "gramag"
copyright = "2024, Thomas Chaplin"
author = "Thomas Chaplin"
release = gramag.version()
version = release


extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.inheritance_diagram",
    "sphinx.ext.autosectionlabel",
    "sphinx.ext.napoleon",
    "sphinx.ext.mathjax",
    "sphinx_rtd_theme",
]

autosummary_generate = True
autosummary_imported_members = True

templates_path = ["_templates"]
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_title = ""
html_theme = "sphinx_rtd_theme"
# html_static_path = ["_static"]
html_static_path = []



from gramag import MagGraph, format_rank_table

# Create your graph based on a square distance matrix
# use -1 to denote an infinite distance

# Undirected cycle graph on N nodes
N = 7
distance_matrix = [[min((j - i) % N, (i - j) % N) for j in range(N)] for i in range(N)]

mg = MagGraph.from_distance_matrix(distance_matrix)
# Compute generators of all MC^{(s, t)}_{k, l} for l<=6
mg.populate_paths(l_max=11)

# Reports the ranks of MC^{(s, t)}_{k, l}, summed over (s, t)
rk_gens = mg.rank_generators()

# For each l, in parallel across each (s, t), computes MH^{(s, t)}_{k, l}
# Adds up the rank for each k, l
rk_hom = mg.rank_homology()

# Pretty print
print("Rank of MC:")
print(format_rank_table(rk_gens))

print("Rank of MH:")
print(format_rank_table(rk_hom))



#!/usr/bin/env python
# coding: utf-8

# This script illustrates how you can use a custom GrPPHATI filtration
# to compute betti_1 of regular path homology (no persistence)

# Pull in deps

import pandas as pd
import networkx as nx
import numpy as np
from pathlib import Path

from gramag import MagGraph, format_rank_table

print("=== CELEGANS ===")

# Read in neurons

# Available here https://www.wormatlas.org/neuronalwiring.html (Section 2.1 in Excel format)
df = pd.read_csv(Path(__file__).parent / "NeuronConnect.csv")
print("Loaded data")

# Just look at synapses from the sending side

sending_types = ["S", "Sp", "EJ"]
sending_df = df[df["Type"].isin(sending_types)]

# Aggregate #synapses between each neuron pair

n_connections_df = (
    sending_df.groupby(["Neuron 1", "Neuron 2"])["Nbr"].sum().reset_index()
)


# Weighted digraph with weight = 1/(number of synapses)

G = nx.DiGraph()
G.add_edges_from(
    (row["Neuron 1"], row["Neuron 2"])  # , {"weight": 1.0 / row["Nbr"]})
    for _, row in n_connections_df.iterrows()
)

# grpphati-rs required integer-labelled nodes
G = nx.convert_node_labels_to_integers(G, label_attribute="Neuron")
print("Built digraph")

mg = MagGraph(G.edges)
l_max = 2
mg.populate_paths(l_max=l_max)
print(f"Populated paths up to l={l_max}")
rk_hom = mg.rank_homology()
print(format_rank_table(rk_hom))

omega_0 = mg.l_homology(0, representatives=True)
print("Got Ω_0")
omega_1 = mg.l_homology(1, representatives=True)
print("Got Ω_1")
omega_2 = mg.l_homology(2, representatives=True)
print("Got Ω_2")



from gramag import MagGraph, format_rank_table

# Create your graph
# A few rules:
# 1. Nodes are labelled by integers
# 2. Edges are provided as a list of tuples of vertices
# 3. Isolated vertices are not supported at the moment
N = 7
mg = MagGraph(
    [(i, (i + 1) % N) for i in range(N)] + [((i + 1) % N, i) for i in range(N)]
)

# Compute generators of all MC^{(s, t)}_{k, l} for l<=6
mg.populate_paths(l_max=11)

# Reports the ranks of MC^{(s, t)}_{k, l}, summed over (s, t)
rk_gens = mg.rank_generators()

# For each l, in parallel across each (s, t), computes MH^{(s, t)}_{k, l}
# Adds up the rank for each k, l
rk_hom = mg.rank_homology()

# Pretty print
print("Rank of MC:")
print(format_rank_table(rk_gens))

print("Rank of MH:")
print(format_rank_table(rk_hom))



from pprint import pprint
import numpy as np
from gramag import MagGraph, format_rank_table
import time


def experiment(N=100, l_max=8):
    # Create graph (tuple of edges, no support for isolated vertices atm)
    mg = MagGraph(
        [(i, (i + 1) % N) for i in range(N)] + [((i + 1) % N, i) for i in range(N)]
    )
    tic = time.time()
    mg.populate_paths(l_max=l_max)
    toc = time.time()
    del mg
    return toc - tic


times = np.array([experiment() for _ in range(50)])
median = np.median(times)
std = np.std(times)
print(f"{median:.2f}±{std:.2f}")
pprint(times)

# print(format_rank_table(mg.rank_generators()))
